package ch.ethz.se.rpnc.tests;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.CharArrayReader;
import java.io.PrintWriter;

import org.junit.Test;
import static org.junit.Assert.assertEquals;

import ch.ethz.se.rpnc.CalculatorController;
import ch.ethz.se.rpnc.ReversePolishNotationCalculator;
import ch.ethz.se.rpnc.ui.CalculatorConsole;

public class ReversePolishNotationCalculatorTestConsole {

	private String testInput;
	private String testOutput;
	private ByteArrayOutputStream outStream;
	private static final String NEW_LINE = System.getProperty("line.separator");

	/**
	 * Test the input against correct output in console mode. testInput contains
	 * the calculator input testOutput contains the correct answer.
	 */
	@Test
	public void testConsoleOperation() throws Exception {
		testInput = "10" + NEW_LINE 
				+ "5" + NEW_LINE 
				+ "+" + NEW_LINE
				+ "evaluate" + NEW_LINE 
				+ "print" + NEW_LINE 
				+ "quit" + NEW_LINE;
		testOutput = "#####15.0" + NEW_LINE 
				+ "#Console calculator quitting..." + NEW_LINE;

		// prepare streams
		BufferedReader reader = new BufferedReader(new CharArrayReader(
				testInput.toCharArray()));
		outStream = new ByteArrayOutputStream();
		PrintWriter writer = new PrintWriter(outStream);

		//<-- You may change the following 5 lines to suit your implementation.
		ReversePolishNotationCalculator calc = new ReversePolishNotationCalculator();	// Model
		CalculatorConsole cons = new CalculatorConsole(reader, writer, calc);			// View
		@SuppressWarnings("unused")
		CalculatorController contr = new CalculatorController(calc, cons);				// Controller
		cons.run();
		//-->
		
		writer.flush();
		assertEquals(testOutput, outStream.toString());
	}

	/**
	 * Test the input against correct output in NLS console mode. testInput
	 * contains the calculator input testOutput contains the correct answer.
	 */
	@Test
	public void testConsoleLanguage() throws Exception {
		// input and output
		testInput = "deutsch" + NEW_LINE 
				+ "10" + NEW_LINE 
				+ "5" + NEW_LINE
				+ "+" + NEW_LINE 
				+ "auswerten" + NEW_LINE 
				+ "drucken" + NEW_LINE 
				+ "Hilfe" + NEW_LINE 
				+ "english" + NEW_LINE 
				+ "help" + NEW_LINE 
				+ "quit" + NEW_LINE;
		testOutput = "#Aktuelle Sprache: deutsch" + NEW_LINE
				+ "Befehle: loeschen, drucken, beenden, hilfe, auswerten" + NEW_LINE 
				+ "#####15.0" + NEW_LINE 
				+ "#Ungueltige Eingabe:" + NEW_LINE 
				+ "\tHilfe" + NEW_LINE
				+ "#Current language: english" + NEW_LINE
				+ "Available commands: clear, print, quit, help, evaluate" + NEW_LINE
				+ "#Available commands: clear, print, quit, help, evaluate"	+ NEW_LINE 
				+ "#Console calculator quitting..." + NEW_LINE;

		// prepare streams
		BufferedReader reader = new BufferedReader(new CharArrayReader(
				testInput.toCharArray()));
		outStream = new ByteArrayOutputStream();
		PrintWriter writer = new PrintWriter(outStream);

		//<-- You may change the following 5 lines to suit your implementation.
		ReversePolishNotationCalculator calc = new ReversePolishNotationCalculator();	// Model
		CalculatorConsole cons = new CalculatorConsole(reader, writer, calc);			// View
		@SuppressWarnings("unused")
		CalculatorController contr = new CalculatorController(calc, cons);				// Controller
		cons.run();
		//-->
		
		writer.flush();
		assertEquals(testOutput, outStream.toString());
	}
}
